package cn.gygxzc.envir.config.shiro.cache

import cn.gygxzc.envir.config.shiro.common.ZenAuthToken
import com.google.gson.Gson
import org.apache.shiro.cache.Cache
import org.springframework.data.redis.core.HashOperations
import org.springframework.data.redis.core.RedisTemplate

/**
 * Created by niantuo on 2018/9/21.
 * 帮助jwt缓存信息的缓存管理
 */

class JwtRedisCache(redisTemplate: RedisTemplate<String, Any>)
    : Cache<String, ZenAuthToken> {

    companion object {
        private const val mCacheKey = "JwtToken"
        private val mGson = Gson()
    }

    private val mHashOperations: HashOperations<String, String, String> = redisTemplate.opsForHash<String, String>()


    override fun values(): MutableCollection<ZenAuthToken?> {
        return keys().map { get(it) }
                .filter { it != null }
                .toMutableList()
    }

    override fun clear() {
        val keys = mHashOperations.keys(mCacheKey)
        keys.forEach { mHashOperations.delete(mCacheKey, it) }
    }

    override fun put(p0: String, p1: ZenAuthToken): ZenAuthToken {
        mHashOperations.put(mCacheKey, p0, mGson.toJson(p1))
        return p1
    }

    override fun remove(p0: String?): ZenAuthToken? {
        if (p0 == null) return null
        val subject = mHashOperations.get(mCacheKey, p0)
        if (subject != null) {
            mHashOperations.delete(mCacheKey, p0)
            return mGson.fromJson<ZenAuthToken>(subject, ZenAuthToken::class.java)
        }
        return null
    }

    override fun size(): Int {
        return mHashOperations.keys(mCacheKey).size
    }

    override fun get(p0: String?): ZenAuthToken? {
        if (p0 == null) return null
        val json = mHashOperations.get(mCacheKey, p0)
        if (json.isNullOrBlank()) return null
        return mGson.fromJson(json, ZenAuthToken::class.java)

    }

    override fun keys(): MutableSet<String> {
        return mHashOperations.keys(mCacheKey)
    }
}