package cn.gygxzc.envir.config.shiro.cache

import cn.gygxzc.envir.config.jwt.JwtConfig
import cn.gygxzc.tina.config.properties.ShiroRedisCacheProperties
import cn.gygxzc.tina.config.properties.ShiroSessionProperties
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.DependsOn
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer
import org.springframework.data.redis.serializer.StringRedisSerializer
import java.util.*
import javax.crypto.spec.SecretKeySpec

/**
 * @program: Zen-Framework
 *
 * @description: ${description}
 *
 * @author: tuonina
 *
 * @create: 2018-09-20 21:41
 **/
@Configuration
@EnableConfigurationProperties(value = [ShiroRedisCacheProperties::class, ShiroSessionProperties::class])
class RedisConfiguration {

    @Autowired
    private lateinit var redisProperties: ShiroRedisCacheProperties

    @Autowired
    private lateinit var sessionProperties: ShiroSessionProperties

    @Bean("shiroRedisConnectionFactory")
    fun connectionFactory(): JedisConnectionFactory {
        return JedisConnectionFactory(redisProperties)
    }


    @Bean
    fun jwtConfig(): JwtConfig {
        val encodedKey = Base64.getEncoder().encode(sessionProperties.jwtKey.toByteArray())
        val key = SecretKeySpec(encodedKey, 0, encodedKey.size, "HmacSHA256")
        return JwtConfig(sessionProperties.expired, key)

    }


    @Bean(name = ["shiroRedisTemplate"])
    fun redisTemplate(): RedisTemplate<String, Any> {
        return RedisTemplate<String, Any>().apply {
            connectionFactory = connectionFactory()
            val stringSerializer = StringRedisSerializer()
            keySerializer = stringSerializer
            valueSerializer = JdkSerializationRedisSerializer()
            hashKeySerializer = stringSerializer
            hashValueSerializer = Jackson2JsonRedisSerializer(Any::class.java)
        }
    }

    @Bean
    fun jwtRedisCache(): JwtRedisCache {
        val template = RedisTemplate<String, Any>()
                .apply {
                    connectionFactory = connectionFactory()
                    val stringSerializer = StringRedisSerializer()
                    keySerializer = stringSerializer
                    valueSerializer = JdkSerializationRedisSerializer()
                    hashKeySerializer = stringSerializer
                    hashValueSerializer = stringSerializer
                }
        template.afterPropertiesSet()
        return JwtRedisCache(template)
    }


    @Bean(name = ["shrioRedisCacheManager"])
    @DependsOn(value = ["shiroRedisTemplate"])
    fun redisCacheManager(): ShiroRedisCacheManager {
        return ShiroRedisCacheManager(redisTemplate(), sessionProperties)
    }


    @Bean
    fun redisSessionDAO(): ShiroRedisSessionDAO {
        return ShiroRedisSessionDAO(redisTemplate(), sessionProperties)
    }


}